package org.openxma.dsl.generator.mojo;

import java.util.Properties;

import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.shared.model.fileset.FileSet;
import org.apache.maven.shared.model.fileset.util.FileSetManager;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.mwe.core.WorkflowContext;
import org.eclipse.emf.mwe.core.WorkflowContextDefaultImpl;
import org.eclipse.emf.mwe.core.issues.IssuesImpl;
import org.eclipse.emf.mwe.core.monitor.NullProgressMonitor;
import org.eclipse.xtext.mwe.NameBasedFilter;
import org.openxma.dsl.common.DslConstants;
import org.openxma.dsl.generator.DefaultGeneratorConfiguration;
import org.openxma.dsl.generator.DefaultGeneratorFactory;
import org.openxma.dsl.generator.Generator;
import org.openxma.dsl.generator.GeneratorConstants;
import org.openxma.dsl.generator.component.PmlReader;
import org.openxma.dsl.pom.PomDslStandaloneSetup;

/**
 * Maven plugin used to execute the openXMA Pom generator workflow
 * 
 * @goal generate-pom
 * @requiresDependencyResolution compile
 * @phase generate-sources
 */
public class PomGeneratorMojo extends AbstractGeneratorMojo {
    /**
     * @required
     * @parameter
     */
    private FileSet componentFileSet;
    /**
     * @parameter default-value="/src/main/resources/guidesign.properties"
     * @required
     * @readonly
     */
    private String guiDesignProperties;

    public void doExecute() throws MojoExecutionException {
        extendCurrentClassloader(getClass().getClassLoader());
        check();
        FileSetManager fileSetManager = new FileSetManager();
        String[] includedFiles = fileSetManager.getIncludedFiles(componentFileSet);

        DefaultGeneratorConfiguration generatorConfiguration = createGeneratorConfiguration(DslConstants.OPENXMA_POM_FILE_EXTENSION);

        Generator generator = new DefaultGeneratorFactory().createAndInit(generatorConfiguration);

        WorkflowContext workflowContext = new WorkflowContextDefaultImpl();
        NullProgressMonitor progressMonitor = new NullProgressMonitor();
        IssuesImpl issues = new IssuesImpl();

        PmlReader pmlReader = new PmlReader();
        pmlReader.addRegister(new PomDslStandaloneSetup());
        pmlReader.setUseJavaClassPath(true);

        NameBasedFilter nameBasedFilter = new NameBasedFilter();
        nameBasedFilter.setRegularExpression("^(?:(?!classes|bin).)*\\.(pml|xma|dml|xmadsl)$");
        pmlReader.setUriFilter(nameBasedFilter);

        for (String srcFolderRelativeModelFile : includedFiles) {
            pmlReader.setPmlUri("classpath:/" + srcFolderRelativeModelFile);
            pmlReader.invoke(workflowContext, progressMonitor, issues);
            EObject eObject = (EObject) workflowContext.get("model");
            generator.generate(eObject.eResource(), progressMonitor);
        }
        getLog().info(includedFiles.length + " files transformed.");
    }

    @Override
    protected Properties loadProperties() throws MojoExecutionException {
        Properties properties = super.loadProperties();
        properties.setProperty(GeneratorConstants.GUIDESIGNER_PROPERTIES_FILE, guiDesignProperties);
        return properties;
    }

    private void check() throws MojoExecutionException {
        if (componentFileSet == null || componentFileSet.getIncludes() == null
                || componentFileSet.getIncludes().size() == 0) {
            throw new MojoExecutionException(
                    "No files specified. User either the 'file' attribute or a 'fileset' element.");
        }
    }

}
